////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                     ***   Arduino Uni-Polar Parametric Switch V 1.0   ***                                      //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Using:                                                                                                                         //
//    Arduino Uno/Duemilanove Compatible MB                                                                                       //
//    ACS712 Current Module                                                                                                       //
//    Optocoupler/Magnetic Isolator/Mosfet                                                                                        //
//    Button with 10K Pull Down Resistor                                                                                          //
//                                                                                                                                //
//   Analog Pins use a Digital to Analog Converter, where Sensor Minimum = 1023 and Sensor Maximum = 0. 1024 bits of precission.  //
//  We are detecting Alternating Current and can detect Current in both polaritys. Positive and Negative. Zero Current is 511.5.  //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                ***   Quatar Wave Switching   ***                                               //
//                           by Chris Sykes - with Contributions from www.aboveunity.com Forum Members.                           //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Define Pins:
const int sensorPin = A0; // Analog Pin Zero.
const int mosfetPin = 3; // Digital Pin Three.
const int buttonPin = 4; // Digital Pin Four.

// Define Sensor's States:
int zeroSensorValue = 511; // 511 is Zero State, or Off. 1023 / 2 = 511.5 to be precise.
int peakSensorValue = 511; // This Value needs to be your Current Peak, Calibration should find this value.
int lastSensorState = LOW; // The Sensor State: LOW is Off HIGH is On (We have passed our Peak).

// Calibration Time in Milliseconds: 10000 = 10 Seconds.
unsigned long seconds = 10000;

// Flags:
bool calibrating = false;


// Default Setup Method:
void setup() {

  // Setup Serial Coms:
  Serial.begin(9600);

  // Pin Mode Setup:
  pinMode(mosfetPin, OUTPUT);
  pinMode(buttonPin, INPUT);
}


// Default Loop Method:
void loop() {

  // Read and Store the Sensor Value:
  int sensorValue = analogRead(sensorPin);

  // Check Button Press:
  int buttonState = digitalRead(buttonPin);

  // Calibrate Once Only, HIGH is Button Press: From Off to On
  if (!calibrating && buttonState == HIGH) {

    // Set calibrating Flag:
    calibrating = true;

    // Turn Off the Mosfet Pin:
    digitalWrite(mosfetPin, LOW);

    // Reset the Peak Sensor Value:
    peakSensorValue = zeroSensorValue;

    // Calibrate:
    Calibrate();
  }

  // Check for Calibration: Should be blocking, but just in case.
  if (!calibrating) {

    // Check for Peak Sensor Value:
    if (sensorValue <= peakSensorValue) {

      // Check we have passed the Peak Sensor Value:
      if (lastSensorState > peakSensorValue) {

        // Turn On the Mosfet Pin:
        digitalWrite(mosfetPin, HIGH);
      }
    }

    // Check for Zero Cross:
    if (sensorValue >= zeroSensorValue) {

      // Turn Off the Mosfet Pin:
      digitalWrite(mosfetPin, LOW);
    }

    // Set Sensor Value for the next Pass:
    lastSensorState = sensorValue;
  }
}


// Calibration Method: Detect Peak Current.
void Calibrate() {

  // The Analog Reads Calibration Time:
  int readCount = 0;

  // Mean Value Peak Samples from Calibration:
  int meanValue = 0;

  // Current Peaks found during Calibration:
  int peakValues = 0;

  // Calculate the Calibration Time:
  unsigned long calibrationTime = millis() + seconds;

  // Calibrate for Five Seconds:
  while (millis() < calibrationTime) {

    // Read and Store the Sensor Value:
    int sensorValue = analogRead(sensorPin);

    // Increment Times Read:
    readCount++;

    // Look for Maximum Sensor Value:
    if (sensorValue < peakSensorValue) {

      // Increment Peak Values:
      peakValues++;

      // Add to the Total:
      meanValue += sensorValue;

      // Keep this value:
      peakSensorValue = sensorValue;
    }
  }

  // Set calibrating Flag:
  calibrating = false;

  // Print Result:
  Serial.println("Calibration complete: ");
  Serial.print("  Sampled: ");
  Serial.print(peakValues);
  Serial.print(" of ");
  Serial.print(readCount);
  Serial.print(" Sensor readings.");
  Serial.print("\r\n  Mean: ");
  Serial.print(meanValue / peakValues);
  Serial.print("\r\n  Peak: ");
  Serial.print(peakSensorValue);
  Serial.print("\r\n");
}

// END Application:
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                 The Current and the Magnetic Field, they are one and the same things. No Current, no Magnetic Field!           //
//          By detecting the Peak of the Current we are effectively detecting the Peak of the Magnetic Field, more or less.       //
//                                                                                                                                //
//        Energy changes in an LC Circuit, from the Capacitor, Energy leaves with the Time Constant as Electrical Energy,         //
//                      at this time the Coil is Transforming Electrical Energy into Magnetic Field Energy.                       //
//                                                                                                                                //
//                  We do not wish to disturb the Electrical Transformation, this is not how a "Generator" works!                 //
//                   We want the Magnetic Field, Changeing in Time, to "Generate" an excess in Electrical Energy!                 //
//                                                                                                                                //
//                       This is from Peak Current to Zero Current, where the Magnetic Field Changes in Time,                     //
//                        Transforming Magnetic Energy back into Electrical Energy to Charge the Capacitor!                       //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
